@@ -14,6 +14,16 @@
 #define HAVE_INTTYPES_H 1
 #define HAVE_STDINT_H 1
 #define HAVE_UNISTD_H 1
+#define __EXTENSIONS__ 1
+#define _ALL_SOURCE 1
+#define _GNU_SOURCE 1
+#define _POSIX_PTHREAD_SEMANTICS 1
+#define _TANDEM_SOURCE 1
+#define __EXTENSIONS__ 1
+#define _ALL_SOURCE 1
+#define _GNU_SOURCE 1
+#define _POSIX_PTHREAD_SEMANTICS 1
+#define _TANDEM_SOURCE 1
 #define HAVE_LONG_LONG 1
 #define HAVE_OFF_T 1
 #define SIZEOF_INT 4
@@ -171,6 +181,6 @@
 #define RUBY_LIB "/usr/local/lib/ruby/1.8"
 #define RUBY_SITE_LIB "/usr/local/lib/ruby/site_ruby"
 #define RUBY_SITE_LIB2 "/usr/local/lib/ruby/site_ruby/1.8"
-#define RUBY_PLATFORM "sparc64-openbsd4.5"
-#define RUBY_ARCHLIB "/usr/local/lib/ruby/1.8/sparc64-openbsd4.5"
-#define RUBY_SITE_ARCHLIB "/usr/local/lib/ruby/site_ruby/1.8/sparc64-openbsd4.5"
+#define RUBY_PLATFORM "sparc64-openbsd4.6"
+#define RUBY_ARCHLIB "/usr/local/lib/ruby/1.8/sparc64-openbsd4.6"
+#define RUBY_SITE_ARCHLIB "/usr/local/lib/ruby/site_ruby/1.8/sparc64-openbsd4.6"
